import java.io.*;

/**
 * UsaProxy - HTTP proxy for tracking and logging of user interactions on websites
 * <br><br>
 * Copyright (C) 2006  Monika Wnuk - Media Informatics Group at the University of Munich
 * <br><br>
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; 
 * either version 2 of the License, or (at your option) any later version.
 * <br><br>
 * Class HTTPTraffic is used for exclusively manage the current HTTP traffic data index that 
 * is used as server response identifier and for store requests and responses to individual 
 * txt-files (according to index).
 */
public class HTTPTraffic {
	
	private int 		index;
	private boolean 	logHttpTraffic;		/**  defines if HTTP traffic is logged or not
    										  *  stored in logHttpTraffic.txt */
	
	/** Constructor, creates an HTTPTraffic. If logging is enabled
	 *  the current HttpTraffic index is retrieved.
     */
	public HTTPTraffic() {
		doHttpTrafficLogging();
		if(logHttpTraffic) this.index = getHttpTrafficIndex(false);
	}
	
	/** returns the current HTTPTraffic index.
	 * 
	 *  @param fromClient defines if an increased index is requested.
	 *  @return the index.
	 *  */
	public synchronized int getHttpTrafficIndex(boolean fromClient) {
		
		byte buffer[] = new byte[80];
		
		int clientIndex = 0;
    	
    	try {
    		
    		/** Open a stream to the HTTP traffic file. */
			FileInputStream fis = new FileInputStream ( "httpTraffic/httpTraffic.txt");

       		int len = fis.read( buffer, 0, fis.available() );
       		String str = new String( buffer, 0, len );
       		
       		clientIndex = Integer.parseInt(str);
       		
			fis.close();

    	}
    	catch ( FileNotFoundException e ) { 
        	/** If httpTraffic file doesn't exist, display error message. */
        	System.err.println("\nAn ERROR occured: httpTraffic file not found:\n");
									e.printStackTrace() ;
									
			try{
				/** Create httpTraffic file. */
				FileOutputStream fos = new FileOutputStream ( "httpTraffic/httpTraffic.txt" , false);
				
				fos.write(("0").getBytes());
				clientIndex = 0;
				fos.flush();
				fos.close();
				System.err.println("httpTraffic file was created with index 0");
			}
			catch ( IOException ie ) { 	
        		System.err.println("\nAn ERROR occured while creating httpTraffic:\n"
							+ ie );
        	}	
			
        }
        
        catch ( IOException ie ) { 	
        	System.err.println("\nAn ERROR occured while getting HTTPTraffic index:\n"
							+ ie );
        }
        
        index = clientIndex;
        if(fromClient) increaseHttpTrafficIndex();
        
        /** notify waiting clients that index file is accessible */
		notifyAll();
        return clientIndex;
    	
    }
    
    /** retrieves a boolean from the logHttpTraffic file which determines if http traffic logging 
     *  has to be performed or not. 
     */
	protected void doHttpTrafficLogging() {
		
		byte buffer[] = new byte[80];
		
		index = 0;
    	
    	try {
    		
    		/** Open a stream to the logHttpTraffic file. */
			FileInputStream fis = new FileInputStream ( "httpTraffic/logHttpTraffic.txt");

       		int len = fis.read( buffer, 0, fis.available() );
       		String str = new String( buffer, 0, len );
       		
       		Boolean doLog 	= new Boolean(str);
       		logHttpTraffic 	= doLog.booleanValue();
       		
			fis.close();

    	}
    	catch ( FileNotFoundException e ) { 
        	/** If httpTraffic file doesn't exist, display error message. */
        	System.err.println("\nAn ERROR occured: httpTraffic file not found:\n"
									+ e );
									
			try{
				/** Create httpTraffic file. */
				FileOutputStream fos = new FileOutputStream ( "httpTraffic/logHttpTraffic.txt" , false);
				
				fos.write(("true").getBytes());
	       		this.logHttpTraffic = true;
				fos.flush();
				fos.close();
				System.err.println("logHttpTraffic file was created with value true");
			}
			catch ( IOException ie ) { 	
        		System.err.println("\nAn ERROR occured while creating httpTraffic:\n"
							+ ie );
        	}	
			
        }
        
        catch ( IOException ie ) { 	
        	System.err.println("\nAn ERROR occured while getting HTTPTraffic index:\n"
							+ ie );
        }
    	
    }
    
    /** increases the serverdata ID (HTTPTraffic index) */
	protected void increaseHttpTrafficIndex() {
    	
    	try {
    		
    		/** Open a stream to the HTTP traffic file. */
			FileOutputStream fos = new FileOutputStream ( "httpTraffic/httpTraffic.txt" , false);

       		fos.write((""+(this.index + 1)).getBytes());
       		this.index += 1;
			fos.flush();
			fos.close();

    	}
    	catch ( FileNotFoundException e ) { 
        	/** If httpTraffic file doesn't exist, display error message. */
        	System.err.println("\nAn ERROR occured: httpTraffic file not found:\n"
									+ e );
        }
        
        catch ( IOException ie ) { 	
        	System.err.println("\nAn ERROR occured while increasing httpTraffic index:\n"
							+ ie );
        }
        /** notify waiting clients that index file is accessible */
		notifyAll();
    	
    }
    
    /** Stores server data to an individual txt-file.
     * 
     *  @param index is the postfix of the txt-file the data has to be stored to 
     *  @param data is the data to be stored
     *  */
	public void store (int index, String data) {
    	
    	try {
    		
    		/** Open a stream to the individual file. */
			FileOutputStream fos = new FileOutputStream ( "httpTraffic/log/httpTraffic"+index+".txt" , true);
			
			fos.write(data.getBytes(HTTPData.HEADER_DEFAULT_CHAR_ENCODING));
			fos.flush();
			fos.close();

    	}
    	catch ( FileNotFoundException e ) { 
        	System.err.println("\nAn ERROR occured: txt file with index " + index + " not found:\n"
									+ e );
        }
        
        catch ( IOException ie ) { 	
        	System.err.println("\nAn ERROR occured while storing data to file with index " + index + ":\n"
							+ ie );
        }
    	
    }
    
	/** Stores server data to an individual txt-file.
     * 
     *  @param index is the postfix of the txt-file the data has to be stored to 
     *  @param data is the byte array a part of which has to be stored
     *  @param off is the start index of the relevant part in the byte array
     *  @param length is the length of the data in the byte array.
     *  */
	public void store (int index, byte[] data, int off, int length) {
    	
    	try {
    		
    		/** Open a stream to the individual file. */
			FileOutputStream fos = new FileOutputStream ( "httpTraffic/log/httpTraffic"+index+".txt" , true);
			
			fos.write(data, off, length);
			fos.flush();
			fos.close();

    	}
    	catch ( FileNotFoundException e ) { 
        	System.err.println("\nAn ERROR occured: txt file with index " + index + " not found:\n"
									+ e );
        }
        
        catch ( IOException ie ) { 	
        	System.err.println("\nAn ERROR occured while storing data to file with index " + index + ":\n"
							+ ie );
        }
    	
    }

	/** Returns the logHttpTraffic.
	 * 
	 *  @return the logHttpTraffic.
	 */
	public boolean isLogHttpTraffic() {
		return logHttpTraffic;
	}

}
